@@ -60,6 +60,9 @@
 #include "wksp_tin_manager.h"
 #include "wksp_grid_manager.h"
 
+//---------------------------------------------------------
+#define FILE_EXISTS(File) (m_Type == Recent_Type::Folder ? wxDirExists(File) : wxFileExists(File))
+
 
 ///////////////////////////////////////////////////////////
 //                                                       //
@@ -70,8 +73,7 @@
 //---------------------------------------------------------
 CWKSP_Data_Menu_File::CWKSP_Data_Menu_File(void)
 {
-	m_DataType	= SG_DATAOBJECT_TYPE_Undefined;
-	m_pMenu		= NULL;
+	m_CmdID[0] = m_CmdID[1] = 0; m_pMenu = NULL;
 }
 
 //---------------------------------------------------------
@@ -98,8 +100,7 @@ void CWKSP_Data_Menu_File::Destroy(void)
 		}
 	}
 
-	m_DataType	= SG_DATAOBJECT_TYPE_Undefined;
-	m_pMenu		= NULL;
+	m_CmdID[0] = m_CmdID[1] = 0; m_pMenu = NULL;
 }
 
 
@@ -108,94 +109,54 @@ void CWKSP_Data_Menu_File::Destroy(void)
 ///////////////////////////////////////////////////////////
 
 //---------------------------------------------------------
-wxMenu * CWKSP_Data_Menu_File::Create(TSG_Data_Object_Type DataType)
+wxMenu * CWKSP_Data_Menu_File::Create(Recent_Type Type)
 {
 	Destroy();
 
-	m_DataType	= DataType;
-
-	m_pMenu	= new wxMenu;
+	m_Type = Type; m_pMenu = new wxMenu;
 
 	//-----------------------------------------------------
-	switch( m_DataType )
+	switch( m_Type )
 	{
-	case SG_DATAOBJECT_TYPE_Undefined:
-		m_Group		= "Project";
-		m_CmdID[0]	= ID_CMD_DATA_PROJECT_RECENT_FIRST;
-		m_CmdID[1]	= ID_CMD_DATA_PROJECT_RECENT_LAST - m_CmdID[0] + 1;
-
+	case Recent_Type::Project:
+		m_Group = "Project";
+		m_CmdID[0] = ID_CMD_DATA_PROJECT_RECENT_FIRST;
+		m_CmdID[1] = ID_CMD_DATA_PROJECT_RECENT_LAST - m_CmdID[0];
 		CMD_Menu_Add_Item(m_pMenu, false, ID_CMD_DATA_PROJECT_OPEN   );
 		CMD_Menu_Add_Item(m_pMenu, false, ID_CMD_DATA_PROJECT_BROWSE );
-		CMD_Menu_Add_Item(m_pMenu, false, ID_CMD_DATA_PROJECT_NEW  );
+		CMD_Menu_Add_Item(m_pMenu, false, ID_CMD_DATA_PROJECT_NEW    );
 		CMD_Menu_Add_Item(m_pMenu, false, ID_CMD_DATA_PROJECT_SAVE   );
 		CMD_Menu_Add_Item(m_pMenu, false, ID_CMD_DATA_PROJECT_SAVE_AS);
 		CMD_Menu_Add_Item(m_pMenu, false, ID_CMD_DATA_PROJECT_COPY   );
 		break;
 
-	case SG_DATAOBJECT_TYPE_Table:
-		m_Group		= "Table";
-		m_CmdID[0]	= ID_CMD_TABLE_RECENT_FIRST;
-		m_CmdID[1]	= ID_CMD_TABLE_RECENT_LAST        - m_CmdID[0] + 1;
-
-		CMD_Menu_Add_Item(m_pMenu, false, ID_CMD_TABLE_OPEN);
+	case Recent_Type::Folder :
+		m_Group = "Folder";
+		m_CmdID[0] = ID_CMD_DATA_FOLDER_RECENT_FIRST;
+		m_CmdID[1] = ID_CMD_DATA_FOLDER_RECENT_LAST - m_CmdID[0];
 		break;
 
-	case SG_DATAOBJECT_TYPE_Shapes:
-		m_Group		= "Shapes";
-		m_CmdID[0]	= ID_CMD_SHAPES_RECENT_FIRST;
-		m_CmdID[1]	= ID_CMD_SHAPES_RECENT_LAST       - m_CmdID[0] + 1;
-
-		CMD_Menu_Add_Item(m_pMenu, false, ID_CMD_SHAPES_OPEN);
-		break;
-
-	case SG_DATAOBJECT_TYPE_TIN:
-		m_Group		= "TIN";
-		m_CmdID[0]	= ID_CMD_TIN_RECENT_FIRST;
-		m_CmdID[1]	= ID_CMD_TIN_RECENT_LAST          - m_CmdID[0] + 1;
-
-		CMD_Menu_Add_Item(m_pMenu, false, ID_CMD_TIN_OPEN);
-		break;
-
-	case SG_DATAOBJECT_TYPE_PointCloud:
-		m_Group		= "Point Cloud";
-		m_CmdID[0]	= ID_CMD_POINTCLOUD_RECENT_FIRST;
-		m_CmdID[1]	= ID_CMD_POINTCLOUD_RECENT_LAST   - m_CmdID[0] + 1;
-
-		CMD_Menu_Add_Item(m_pMenu, false, ID_CMD_POINTCLOUD_OPEN);
-		break;
-
-	case SG_DATAOBJECT_TYPE_Grid:
-		m_Group		= "Grid";
-		m_CmdID[0]	= ID_CMD_GRID_RECENT_FIRST;
-		m_CmdID[1]	= ID_CMD_GRID_RECENT_LAST         - m_CmdID[0] + 1;
-
-		CMD_Menu_Add_Item(m_pMenu, false, ID_CMD_GRID_OPEN);
-		break;
-
-	case SG_DATAOBJECT_TYPE_Grids:
-		m_Group		= "Grids";
-		m_CmdID[0]	= ID_CMD_GRIDS_RECENT_FIRST;
-		m_CmdID[1]	= ID_CMD_GRIDS_RECENT_LAST        - m_CmdID[0] + 1;
-
-		CMD_Menu_Add_Item(m_pMenu, false, ID_CMD_GRIDS_OPEN);
+	case Recent_Type::Data   :
+		m_Group = "Data";
+		m_CmdID[0] = ID_CMD_DATA_FILE_RECENT_FIRST;
+		m_CmdID[1] = ID_CMD_DATA_FILE_RECENT_LAST - m_CmdID[0];
 		break;
 
 	default:
-		m_CmdID[0]	= 0;
-		m_CmdID[1]	= 0;
+		m_CmdID[1] = m_CmdID[0] = 0;
 		break;
 	}
 
 	//-----------------------------------------------------
-	m_Offset	= m_pMenu->GetMenuItemCount();
+	m_Offset = m_pMenu->GetMenuItemCount();
 
 	if( m_CmdID[0] )
 	{
-		int	i = 0; wxString File;
+		int i = 0; wxString File;
 
 		while( CONFIG_Read("RECENT_FILES/" + m_Group, wxString::Format("FILE_%02d", ++i), File) )
 		{
-			bool bAdd = wxFileExists(File);
+			bool bAdd = FILE_EXISTS(File);
 
 			for(size_t j=0; bAdd && j<m_Files.Count(); j++)
 			{
@@ -231,7 +192,7 @@ void CWKSP_Data_Menu_File::Update(void)
 	//-----------------------------------------------------
 	for(int i=m_Files.GetCount(); i>0; i--)
 	{
-		if( !wxFileExists(m_Files[i - 1]) )
+		if( !FILE_EXISTS(m_Files[i - 1]) )
 		{
 			m_Files.RemoveAt(i - 1);
 		}
@@ -279,11 +240,11 @@ void CWKSP_Data_Menu_File::Update(void)
 ///////////////////////////////////////////////////////////
 
 //---------------------------------------------------------
-void CWKSP_Data_Menu_File::Add(const wxString &File)
+void CWKSP_Data_Menu_File::Add(const wxString &File, bool bUpdate)
 {
-	Del(File);
+	Del(File, false);
 
-	if( wxFileExists(File) )
+	if( FILE_EXISTS(File) )
 	{
 		m_Files.Insert(File, 0);
 
@@ -292,10 +253,15 @@ void CWKSP_Data_Menu_File::Add(const wxString &File)
 			m_Files.RemoveAt(m_CmdID[1]);
 		}
 	}
+
+	if( bUpdate )
+	{
+		Update();
+	}
 }
 
 //---------------------------------------------------------
-void CWKSP_Data_Menu_File::Del(const wxString &File)
+void CWKSP_Data_Menu_File::Del(const wxString &File, bool bUpdate)
 {
 	for(size_t i=m_Files.GetCount(); i>0; i--)
 	{
@@ -304,6 +270,11 @@ void CWKSP_Data_Menu_File::Del(const wxString &File)
 			m_Files.RemoveAt(i - 1);
 		}
 	}
+
+	if( bUpdate )
+	{
+		Update();
+	}
 }
 
 //---------------------------------------------------------
@@ -316,7 +287,7 @@ bool CWKSP_Data_Menu_File::Get(wxArrayString &Files, bool bAppend)
 
 	for(size_t i=0; i<m_Files.GetCount(); i++)
 	{
-		if( wxFileExists(m_Files[i]) )
+		if( FILE_EXISTS(m_Files[i]) )
 		{
 			Files.Add(m_Files[i]);
 		}
@@ -325,6 +296,12 @@ bool CWKSP_Data_Menu_File::Get(wxArrayString &Files, bool bAppend)
 	return( Files.Count() > 0 );
 }
 
+//---------------------------------------------------------
+int CWKSP_Data_Menu_File::Count(void)
+{
+	return( m_Files.GetCount() );
+}
+
 
 ///////////////////////////////////////////////////////////
 //                                                       //
@@ -333,31 +310,19 @@ bool CWKSP_Data_Menu_File::Get(wxArrayString &Files, bool bAppend)
 //---------------------------------------------------------
 bool CWKSP_Data_Menu_File::Open(int CmdID)
 {
-	bool bSuccess = false; int i = CmdID - m_CmdID[0];
+	int i = CmdID - m_CmdID[0];
 
 	if( i >= 0 && i < (int)m_Files.GetCount() )
 	{
-		switch( m_DataType )
+		if( m_Type == Recent_Type::Folder )
 		{
-		case SG_DATAOBJECT_TYPE_Undefined:
-			bSuccess	= g_pData->Open(wxString(m_Files[i]));
-			break;
-
-		case SG_DATAOBJECT_TYPE_Table     :
-		case SG_DATAOBJECT_TYPE_Shapes    :
-		case SG_DATAOBJECT_TYPE_TIN       :
-		case SG_DATAOBJECT_TYPE_PointCloud:
-		case SG_DATAOBJECT_TYPE_Grid      :
-		case SG_DATAOBJECT_TYPE_Grids     :
-			bSuccess	= g_pData->Open(wxString(m_Files[i]), m_DataType) != NULL;
-			break;
-
-		default:
-			break;
+			return( g_pData->Open_Directory(m_Files[i]) );
 		}
+
+		return( g_pData->Open(wxString(m_Files[i])) );
 	}
 
-	return( bSuccess );
+	return( false );
 }
 
 
