@@ -47,11 +47,8 @@
 
 //---------------------------------------------------------
 #include "res_commands.h"
-#include "res_controls.h"
 #include "res_dialogs.h"
 
-#include "saga_frame.h"
-
 #include "helper.h"
 
 #include "active.h"
@@ -136,14 +133,11 @@ CWKSP_Layer::CWKSP_Layer(CSG_Data_Object *pObject)
 //---------------------------------------------------------
 CWKSP_Layer::~CWKSP_Layer(void)
 {
-	if( g_pSAGA_Frame ) { g_pSAGA_Frame->Freeze(); }
-
-	if( g_pMaps       ) { g_pMaps->Del(this);      }
-
-	if( m_pClassify   ) { delete(m_pClassify);     }
-	if( m_pLegend     ) { delete(m_pLegend  );     }
-
-	if( g_pSAGA_Frame ) { g_pSAGA_Frame->Thaw();   }
+	MDI_Freeze();
+	if( g_pMaps     ) { g_pMaps->Del(this);  }
+	if( m_pClassify ) { delete(m_pClassify); }
+	if( m_pLegend   ) { delete(m_pLegend  ); }
+	MDI_Thaw();
 }
 
 
@@ -156,14 +150,10 @@ bool CWKSP_Layer::On_Command(int Cmd_ID)
 {
 	switch( Cmd_ID )
 	{
-	default:
-		return( CWKSP_Data_Item::On_Command(Cmd_ID) );
+	default: return( CWKSP_Data_Item::On_Command(Cmd_ID) );
 
 	case ID_CMD_WKSP_ITEM_RETURN:
-	case ID_CMD_SHAPES_SHOW     :
-	case ID_CMD_GRID_SHOW       :
-	case ID_CMD_TIN_SHOW        :
-	case ID_CMD_POINTCLOUD_SHOW :
+	case ID_CMD_DATA_SHOW_MAP   :
 		g_pMaps->Add(this);
 		break;
 
@@ -915,19 +905,20 @@ CSG_Rect CWKSP_Layer::Get_Extent(void)
 //---------------------------------------------------------
 void CWKSP_Layer::_Set_Projection(void)
 {
-	CSG_Projection Projection(Get_Object()->Get_Projection());
+	CSG_Projection &CRS = Get_Object()->Get_Projection();
 
-	CSG_Tool *pTool = SG_Get_Tool_Library_Manager().Create_Tool("pj_proj4", 15, true);	// CCRS_Picker
+	CSG_Tool *pTool = SG_Get_Tool_Library_Manager().Create_Tool("pj_proj4", 15, true); // CCRS_Picker
 
-	if(	pTool && pTool->Set_Parameter("CRS_STRING", Projection.Get_WKT())
+	if(	pTool
+	&&  pTool->Set_Parameter("CRS_WKT" , CRS.Get_WKT2())
+	&&  pTool->Set_Parameter("CRS_PROJ", CRS.Get_PROJ())
 	&&	pTool->On_Before_Execution() && DLG_Parameters(pTool->Get_Parameters()) )
 	{
-		Projection.Create(pTool->Get_Parameter("CRS_WKT")->asString());
+		CSG_Projection new_CRS(pTool->Get_Parameter("CRS_WKT")->asString(), pTool->Get_Parameter("CRS_PROJ")->asString());
 
-		if( Projection.is_Okay() && !Projection.is_Equal(Get_Object()->Get_Projection()) )
+		if( new_CRS.is_Okay() && (new_CRS != CRS || new_CRS.Get_Code() != CRS.Get_Code() || new_CRS.Get_Authority().CmpNoCase(CRS.Get_Authority())) )
 		{
-			Get_Object()->Get_Projection().Create(Projection);
-			Get_Object()->Set_Modified();
+			CRS.Create(new_CRS); Get_Object()->Set_Modified();
 
 			DataObject_Changed();
 		}
@@ -1095,9 +1086,8 @@ bool CWKSP_Layer::Show(int Flags)
 	case SG_UI_DATAOBJECT_SHOW_MAP_LAST:
 		return( Show(g_pMaps->Get_Map(g_pMaps->Get_Count() - 1)) );
 
-	case SG_UI_DATAOBJECT_SHOW_MAP_ACTIVE: {
-		CVIEW_Map *pView = (CVIEW_Map *)g_pSAGA_Frame->Get_Active_Child(ID_VIEW_MAP);
-		return( Show(pView ? pView->Get_Map() : NULL) ); }
+	case SG_UI_DATAOBJECT_SHOW_MAP_ACTIVE:
+		return( Show(MDI_Get_Active_Map() ? ((CVIEW_Map *)MDI_Get_Active_Map())->Get_Map() : NULL) );
 	}
 
 	return( false );
